@@ -3,44 +3,32 @@
 
 {-
 
-  BSD LICENSE
+  MIT License
 
-  Copyright (c) 2017, Michael Truog <mjtruog at gmail dot com>
-  All rights reserved.
+  Copyright (c) 2017-2019 Michael Truog <mjtruog at protonmail dot com>
 
-  Redistribution and use in source and binary forms, with or without
-  modification, are permitted provided that the following conditions are met:
+  Permission is hereby granted, free of charge, to any person obtaining a
+  copy of this software and associated documentation files (the "Software"),
+  to deal in the Software without restriction, including without limitation
+  the rights to use, copy, modify, merge, publish, distribute, sublicense,
+  and/or sell copies of the Software, and to permit persons to whom the
+  Software is furnished to do so, subject to the following conditions:
 
-      * Redistributions of source code must retain the above copyright
-        notice, this list of conditions and the following disclaimer.
-      * Redistributions in binary form must reproduce the above copyright
-        notice, this list of conditions and the following disclaimer in
-        the documentation and/or other materials provided with the
-        distribution.
-      * All advertising materials mentioning features or use of this
-        software must display the following acknowledgment:
-          This product includes software developed by Michael Truog
-      * The name of the author may not be used to endorse or promote
-        products derived from this software without specific prior
-        written permission
+  The above copyright notice and this permission notice shall be included in
+  all copies or substantial portions of the Software.
 
-  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
-  CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
-  INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
-  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
-  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
-  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
-  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
-  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
-  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
-  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
-  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
-  DAMAGE.
+  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+  DEALINGS IN THE SOFTWARE.
 
  -}
 
+-- | Erlang Binary Term Format Encoding/Decoding
+
 module Foreign.Erlang
     ( OtpErlangTerm(..)
     , Error(..)
@@ -49,7 +37,7 @@ module Foreign.Erlang
     , termToBinary
     ) where
 
-import Prelude hiding (length,tail)
+import Prelude hiding (length,tail,(<>))
 import Data.Bits ((.&.))
 import Control.Monad (replicateM)
 import qualified Data.Monoid as Monoid
@@ -92,6 +80,12 @@ tagBitBinaryExt :: Word8
 tagBitBinaryExt = 77
 tagAtomCacheRef :: Word8
 tagAtomCacheRef = 78
+tagNewPidExt :: Word8
+tagNewPidExt = 88
+tagNewPortExt :: Word8
+tagNewPortExt = 89
+tagNewerReferenceExt :: Word8
+tagNewerReferenceExt = 90
 tagSmallIntegerExt :: Word8
 tagSmallIntegerExt = 97
 tagIntegerExt :: Word8
@@ -203,6 +197,7 @@ infixr 4 <>
 (<>) :: Monoid.Monoid m => m -> m -> m
 (<>) = Monoid.mappend
 
+-- | Decode Erlang terms within binary data into Haskell types
 binaryToTerm :: LazyByteString -> Result OtpErlangTerm
 binaryToTerm binary =
     let size = LazyByteString.length binary in
@@ -217,6 +212,7 @@ binaryToTerm binary =
             Right (_, _, term) ->
                 ok term
 
+-- | Encode Haskell types into Erlang terms in binary data
 termToBinary :: OtpErlangTerm -> Int -> Result LazyByteString
 termToBinary term compressed
     | compressed < (-1) || compressed > 9 =
@@ -274,23 +270,24 @@ binaryToTerms = do
             j <- Get.getWord16be
             value <- Get.getByteString $ getUnsignedInt16 j
             return $ OtpErlangAtom value
-        | tag == tagReferenceExt || tag == tagPortExt -> do
+        | tag == tagNewPortExt ||
+          tag == tagReferenceExt || tag == tagPortExt -> do
             (nodeTag, node) <- binaryToAtom
             eid <- Get.getByteString 4
-            creation <- Get.getWord8
+            let creationSize = if tag == tagNewPortExt then 4 else 1
+            creation <- Get.getByteString creationSize
             if tag == tagReferenceExt then
                 return $ OtpErlangReference $ E.Reference
                     nodeTag node eid creation
-            else if tag == tagPortExt then
+            else -- tag == tagNewPortExt || tag == tagPortExt
                 return $ OtpErlangPort $ E.Port
                     nodeTag node eid creation
-            else
-                fail $ "invalid"
-        | tag == tagPidExt -> do
+        | tag == tagNewPidExt || tag == tagPidExt -> do
             (nodeTag, node) <- binaryToAtom
             eid <- Get.getByteString 4
             serial <- Get.getByteString 4
-            creation <- Get.getWord8
+            let creationSize = if tag == tagNewPidExt then 4 else 1
+            creation <- Get.getByteString creationSize
             return $ OtpErlangPid $ E.Pid
                 nodeTag node eid serial creation
         | tag == tagSmallTupleExt || tag == tagLargeTupleExt -> do
@@ -335,10 +332,11 @@ binaryToTerms = do
             value <- Get.getByteString length
             return $ OtpErlangFunction $ E.Function
                 tag value
-        | tag == tagNewReferenceExt -> do
+        | tag == tagNewerReferenceExt || tag == tagNewReferenceExt -> do
             j <- Get.getWord16be
             (nodeTag, node) <- binaryToAtom
-            creation <- Get.getWord8
+            let creationSize = if tag == tagNewerReferenceExt then 4 else 1
+            creation <- Get.getByteString creationSize
             eid <- Get.getByteString $ (getUnsignedInt16 j) * 4
             return $ OtpErlangReference $ E.Reference
                 nodeTag node eid creation
@@ -438,11 +436,18 @@ binaryToPid :: Get Pid
 binaryToPid = do
     tag <- Get.getWord8
     case () of
-      _ | tag == tagPidExt -> do
+      _ | tag == tagNewPidExt -> do
             (nodeTag, node) <- binaryToAtom
             eid <- Get.getByteString 4
             serial <- Get.getByteString 4
-            creation <- Get.getWord8
+            creation <- Get.getByteString 4
+            return $ E.Pid
+                nodeTag node eid serial creation
+        | tag == tagPidExt -> do
+            (nodeTag, node) <- binaryToAtom
+            eid <- Get.getByteString 4
+            serial <- Get.getByteString 4
+            creation <- Get.getByteString 1
             return $ E.Pid
                 nodeTag node eid serial creation
         | otherwise ->
@@ -661,20 +666,30 @@ termsToBinary (OtpErlangMap value) =
     else
         errorType $ OutputError "uint32 overflow"
 termsToBinary (OtpErlangPid (E.Pid nodeTag node eid serial creation)) =
+    let tag =
+            if (ByteString.length creation) == 4 then 
+                tagNewPidExt
+            else
+                tagPidExt in
     ok $ Builder.toLazyByteString $
-        Builder.word8 tagPidExt <>
+        Builder.word8 tag <>
         Builder.word8 nodeTag <>
         Builder.byteString node <>
         Builder.byteString eid <>
         Builder.byteString serial <>
-        Builder.word8 creation
+        Builder.byteString creation
 termsToBinary (OtpErlangPort (E.Port nodeTag node eid creation)) =
+    let tag =
+            if (ByteString.length creation) == 4 then 
+                tagNewPortExt
+            else
+                tagPortExt in
     ok $ Builder.toLazyByteString $
-        Builder.word8 tagPortExt <>
+        Builder.word8 tag <>
         Builder.word8 nodeTag <>
         Builder.byteString node <>
         Builder.byteString eid <>
-        Builder.word8 creation
+        Builder.byteString creation
 termsToBinary (OtpErlangReference (E.Reference nodeTag node eid creation)) =
     let length = (ByteString.length eid) `quot` 4 in
     if length == 0 then
@@ -683,14 +698,19 @@ termsToBinary (OtpErlangReference (E.Reference nodeTag node eid creation)) =
             Builder.word8 nodeTag <>
             Builder.byteString node <>
             Builder.byteString eid <>
-            Builder.word8 creation
+            Builder.byteString creation
     else if length <= 65535 then
+        let tag =
+                if (ByteString.length creation) == 4 then 
+                    tagNewerReferenceExt
+                else
+                    tagNewReferenceExt in
         ok $ Builder.toLazyByteString $
-            Builder.word8 tagNewReferenceExt <>
+            Builder.word8 tag <>
             Builder.word16BE (fromIntegral length) <>
             Builder.word8 nodeTag <>
             Builder.byteString node <>
-            Builder.word8 creation <>
+            Builder.byteString creation <>
             Builder.byteString eid
     else
         errorType $ OutputError "uint16 overflow"
