@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2014-2020 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2014-2021 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -91,7 +91,7 @@ if (Erlang.nodejs_version_after('10.0.0',true)) {
             //
             // Until the exception handling of the domain module is
             // provided by a different module (like async_hooks) or
-            // exceptions (throw/try/catch) are removed from Javascript,
+            // exceptions (throw/try/catch) are removed from JavaScript,
             // the use of the domain module needs to remain
             // (to remain consistent with other CloudI API implementations).
             // The stderr runtime deprecation information is ignored
@@ -184,6 +184,17 @@ TerminateException.prototype = Object.create(Error.prototype, {
 TerminateException.prototype.timeout = function () {
     return this._timeout;
 };
+var FatalError = function FatalError (message) {
+    var error = new Error(message);
+    error.name = 'FatalError';
+    this.message = error.message;
+    if (error.stack) {
+        this.stack = error.stack;
+    }
+}
+FatalError.prototype = Object.create(Error.prototype, {
+    name: { value: 'FatalError' }
+});
 
 CloudI.stdout_write = function stdout_write (s) {
     fs.writeSync(1, s);
@@ -215,6 +226,7 @@ CloudI.API = function API (thread_index, callback) {
                callback : function (API_) {};
     var protocol_str = process.env.CLOUDI_API_INIT_PROTOCOL;
     if (protocol_str === undefined) {
+        CloudI.stderr_write('CloudI service execution must occur in CloudI\n');
         throw new InvalidInputException();
     }
     var buffer_size_str = process.env.CLOUDI_API_INIT_BUFFER_SIZE;
@@ -549,6 +561,10 @@ CloudI.API.prototype.timeout_terminate = function () {
     return this._timeout_terminate;
 };
 
+CloudI.API.prototype.priority_default = function () {
+    return this._priority_default;
+};
+
 CloudI.API.prototype._null_response = function (request_type, name, pattern,
                                                 request_info, request,
                                                 timeout, priority,
@@ -597,6 +613,11 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                     API._terminate = true;
                     CloudI.stderr_write(err.stack + '\n');
                 }
+                else if (err instanceof assert.AssertionError ||
+                         err instanceof FatalError) {
+                    CloudI.stderr_write(err.stack + '\n');
+                    process.exit(1);
+                }
                 else {
                     return_null_response = true;
                     API._exception(err);
@@ -679,6 +700,11 @@ CloudI.API.prototype._callback = function (command, name, pattern,
                 else if (err instanceof ForwardSyncException) {
                     return;
                 }
+                else if (err instanceof assert.AssertionError ||
+                         err instanceof FatalError) {
+                    CloudI.stderr_write(err.stack + '\n');
+                    process.exit(1);
+                }
                 else {
                     return_null_response = true;
                     API._exception(err);
@@ -1161,5 +1187,6 @@ CloudI.ForwardSyncException = ForwardSyncException;
 CloudI.ForwardAsyncException = ForwardAsyncException;
 CloudI.MessageDecodingException = MessageDecodingException;
 CloudI.TerminateException = TerminateException;
+CloudI.FatalError = FatalError;
 
 };
