@@ -1,4 +1,4 @@
-%% Copyright (c) 2012-2014, Loïc Hoguin <essen@ninenines.eu>
+%% Copyright (c) 2012-2018, Loïc Hoguin <essen@ninenines.eu>
 %%
 %% Permission to use, copy, modify, and/or distribute this software for any
 %% purpose with or without fee is hereby granted, provided that the above
@@ -20,4 +20,5 @@
 		Socket::any(),
 		Transport::module(),
 		ProtocolOptions::any())
-	-> {ok, ConnectionPid::pid()}.
+	-> {ok, ConnectionPid::pid()}
+	| {ok, SupPid::pid(), ConnectionPid::pid()}.
