@@ -9,7 +9,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2015-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2015-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -30,8 +30,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2015-2018 Michael Truog
-%%% @version 1.7.5 {@date} {@time}
+%%% @copyright 2015-2020 Michael Truog
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_args_type).
@@ -41,10 +41,14 @@
 -export([function_required/2,
          function_required_pick/2,
          function_optional/2,
+         priority/1,
+         service_name/1,
          service_name_suffix/2,
-         service_name_pattern_suffix/2]).
+         service_name_pattern_suffix/2,
+         timeout_milliseconds/1]).
 
--include_lib("cloudi_core/include/cloudi_logger.hrl").
+-include("cloudi_logger.hrl").
+-include("cloudi_core_i_constants.hrl").
 
 %%%------------------------------------------------------------------------
 %%% External interface functions
@@ -63,12 +67,12 @@ function_required({{M, F}}, Arity)
                 is_function(Function) ->
                     function_required(Function, Arity);
                 true ->
-                    ?LOG_ERROR_SYNC("function ~w:~w/~w does not "
+                    ?LOG_ERROR_SYNC("function ~w:~tw/~w does not "
                                     "return a function!", [M, F, 0]),
                     erlang:exit(badarg)
             end;
         false ->
-            ?LOG_ERROR_SYNC("function ~w:~w/~w does not exist!", [M, F, 0]),
+            ?LOG_ERROR_SYNC("function ~w:~tw/~w does not exist!", [M, F, 0]),
             erlang:exit(badarg)
     end;
 function_required({M, F}, Arity)
@@ -77,7 +81,7 @@ function_required({M, F}, Arity)
         true ->
             fun M:F/Arity;
         false ->
-            ?LOG_ERROR_SYNC("function ~w:~w/~w does not exist!",
+            ?LOG_ERROR_SYNC("function ~w:~tw/~w does not exist!",
                             [M, F, Arity]),
             erlang:exit(badarg)
     end;
@@ -91,7 +95,7 @@ function_required(Function, Arity)
             erlang:exit(badarg)
     end;
 function_required(Function, _) ->
-    ?LOG_ERROR_SYNC("not a function: ~p", [Function]),
+    ?LOG_ERROR_SYNC("not a function: ~tp", [Function]),
     erlang:exit(badarg).
 
 -spec function_required_pick({{module(), atom()}} | {module(), atom()} | fun(),
@@ -107,12 +111,12 @@ function_required_pick({{M, F}}, [_ | _] = ArityOrder)
                 is_function(Function) ->
                     function_required_pick(Function, ArityOrder);
                 true ->
-                    ?LOG_ERROR_SYNC("function ~w:~w/~w does not "
+                    ?LOG_ERROR_SYNC("function ~w:~tw/~w does not "
                                     "return a function!", [M, F, 0]),
                     erlang:exit(badarg)
             end;
         false ->
-            ?LOG_ERROR_SYNC("function ~w:~w/~w does not exist!", [M, F, 0]),
+            ?LOG_ERROR_SYNC("function ~w:~tw/~w does not exist!", [M, F, 0]),
             erlang:exit(badarg)
     end;
 function_required_pick({M, F}, [_ | _] = ArityOrder)
@@ -122,7 +126,7 @@ function_required_pick(Function, [_ | _] = ArityOrder)
     when is_function(Function) ->
     function_required_pick_function(ArityOrder, Function, ArityOrder);
 function_required_pick(Function, [_ | _]) ->
-    ?LOG_ERROR_SYNC("not a function: ~p", [Function]),
+    ?LOG_ERROR_SYNC("not a function: ~tp", [Function]),
     erlang:exit(badarg).
 
 -spec function_optional(undefined |
@@ -135,6 +139,35 @@ function_optional(undefined, _) ->
 function_optional(Function, Arity) ->
     function_required(Function, Arity).
 
+-spec priority(Priority :: cloudi:priority()) ->
+    true.
+
+priority(undefined) ->
+    true;
+priority(Priority)
+    when is_integer(Priority),
+         Priority >= ?PRIORITY_HIGH, Priority =< ?PRIORITY_LOW ->
+    true;
+priority(Priority) ->
+    ?LOG_ERROR_SYNC("invalid priority: ~tp", [Priority]),
+    erlang:exit(badarg).
+
+-spec service_name(Name :: cloudi:service_name()) ->
+    true.
+
+service_name([NameC | _] = Name)
+    when is_integer(NameC) ->
+    case cloudi_x_trie:is_pattern2(Name) of
+        true ->
+            ?LOG_ERROR_SYNC("service name is pattern: \"~ts\"", [Name]),
+            erlang:exit(badarg);
+        false ->
+            true
+    end;
+service_name(Name) ->
+    ?LOG_ERROR_SYNC("invalid service name: ~tp", [Name]),
+    erlang:exit(badarg).
+
 -spec service_name_suffix(Prefix :: cloudi:service_name_pattern(),
                           Name :: cloudi:service_name()) ->
     string().
@@ -143,13 +176,13 @@ service_name_suffix([PrefixC | _] = Prefix, [NameC | _] = Name)
     when is_integer(PrefixC), is_integer(NameC) ->
     case cloudi_x_trie:is_pattern2(Name) of
         true ->
-            ?LOG_ERROR_SYNC("service name is pattern: \"~s\"", [Name]),
+            ?LOG_ERROR_SYNC("service name is pattern: \"~ts\"", [Name]),
             erlang:exit(badarg);
         false ->
             case cloudi_x_trie:pattern2_suffix(Prefix, Name) of
                 error ->
                     ?LOG_ERROR_SYNC("prefix service name mismatch: "
-                                    "\"~s\" \"~s\"", [Prefix, Name]),
+                                    "\"~ts\" \"~ts\"", [Prefix, Name]),
                     erlang:exit(badarg);
                 Suffix ->
                     Suffix
@@ -157,11 +190,11 @@ service_name_suffix([PrefixC | _] = Prefix, [NameC | _] = Name)
     end;
 service_name_suffix([PrefixC | _], Name)
     when is_integer(PrefixC) ->
-    ?LOG_ERROR_SYNC("invalid service name: ~p", [Name]),
+    ?LOG_ERROR_SYNC("invalid service name: ~tp", [Name]),
     erlang:exit(badarg);
 service_name_suffix(Prefix, [NameC | _])
     when is_integer(NameC) ->
-    ?LOG_ERROR_SYNC("invalid prefix: ~p", [Prefix]),
+    ?LOG_ERROR_SYNC("invalid prefix: ~tp", [Prefix]),
     erlang:exit(badarg).
 
 -spec service_name_pattern_suffix(Prefix :: cloudi:service_name_pattern(),
@@ -173,18 +206,34 @@ service_name_pattern_suffix([PrefixC | _] = Prefix, [PatternC | _] = Pattern)
     case suffix_pattern_parse(Prefix, Pattern) of
         error ->
             ?LOG_ERROR_SYNC("prefix service name pattern mismatch: "
-                            "\"~s\" \"~s\"", [Prefix, Pattern]),
+                            "\"~ts\" \"~ts\"", [Prefix, Pattern]),
             erlang:exit(badarg);
         Suffix ->
             Suffix
     end;
 service_name_pattern_suffix([PrefixC | _], Pattern)
     when is_integer(PrefixC) ->
-    ?LOG_ERROR_SYNC("invalid service name pattern: ~p", [Pattern]),
+    ?LOG_ERROR_SYNC("invalid service name pattern: ~tp", [Pattern]),
     erlang:exit(badarg);
 service_name_pattern_suffix(Prefix, [PatternC | _])
     when is_integer(PatternC) ->
-    ?LOG_ERROR_SYNC("invalid prefix: ~p", [Prefix]),
+    ?LOG_ERROR_SYNC("invalid prefix: ~tp", [Prefix]),
+    erlang:exit(badarg).
+
+-spec timeout_milliseconds(Timeout :: cloudi:timeout_milliseconds()) ->
+    true.
+
+timeout_milliseconds(undefined) ->
+    true;
+timeout_milliseconds(limit_min) ->
+    true;
+timeout_milliseconds(limit_max) ->
+    true;
+timeout_milliseconds(Timeout)
+    when is_integer(Timeout), Timeout >= 0, Timeout =< ?TIMEOUT_MAX_ERLANG ->
+    true;
+timeout_milliseconds(Timeout) ->
+    ?LOG_ERROR_SYNC("invalid timeout: ~tp", [Timeout]),
     erlang:exit(badarg).
 
 %%%------------------------------------------------------------------------
@@ -199,7 +248,7 @@ function_required_pick_module([Arity | ArityL], M, F, ArityOrder)
         false ->
             if
                 ArityL == [] ->
-                    ?LOG_ERROR_SYNC("function ~w:~w/~w does not exist!",
+                    ?LOG_ERROR_SYNC("function ~w:~tw/~w does not exist!",
                                     [M, F, ArityOrder]),
                     erlang:exit(badarg);
                 true ->
