@@ -31,7 +31,7 @@
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
 %%% @copyright 2011-2020 Michael Truog
-%%% @version 1.8.1 {@date} {@time}
+%%% @version 2.0.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_core_i_configurator).
@@ -40,7 +40,7 @@
 -behaviour(gen_server).
 
 %% external interface
--export([start_link/1,
+-export([start_link/2,
          configure/0,
          acl_add/2,
          acl_remove/2,
@@ -67,6 +67,9 @@
          logging_formatters_set/2,
          logging_redirect_set/2,
          logging/1,
+         code_path_add/2,
+         code_path_remove/2,
+         code_path/1,
          code_status/3,
          service_start/2,
          service_stop/3,
@@ -126,7 +129,7 @@
 -type error_reason_nodes_set() ::
     {set_invalid, any()}.
 -type error_reason_code_status() ::
-    {file, {file:filename(), file:posix() | badarg}}.
+    {file, {cloudi_service_api:file_path(), file:posix() | badarg}}.
 -export_type([error_reason_service_start/0,
               error_reason_service_stop/0,
               error_reason_service_restart/0,
@@ -141,12 +144,10 @@
     {
         node_name :: nonempty_string(),
         time_start :: cloudi_timestamp:native_monotonic(),
+        terminate_timeout :: pos_integer(),
         configuration :: #config{}
     }).
 
--define(CATCH_EXIT(F),
-        try F catch exit:{Reason, _} -> {error, Reason} end).
-
 -ifdef(CLOUDI_CORE_STANDALONE).
 -dialyzer({no_match,
            [service_update_external/6]}).
@@ -156,8 +157,9 @@
 %%% External interface functions
 %%%------------------------------------------------------------------------
 
-start_link(#config{} = Config) ->
-    gen_server:start_link({local, ?MODULE}, ?MODULE, [Config], []).
+start_link(TerminateTimeout, #config{} = Config) ->
+    gen_server:start_link({local, ?MODULE}, ?MODULE,
+                          [TerminateTimeout, Config], []).
 
 configure() ->
     gen_server:call(?MODULE, configure, infinity).
@@ -258,6 +260,17 @@ logging_redirect_set(L, Timeout) ->
 logging(Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE, logging, Timeout)).
 
+code_path_add(Path, Timeout) ->
+    ?CATCH_EXIT(gen_server:call(?MODULE,
+                                {code_path_add, Path}, Timeout)).
+
+code_path_remove(Path, Timeout) ->
+    ?CATCH_EXIT(gen_server:call(?MODULE,
+                                {code_path_remove, Path}, Timeout)).
+
+code_path(Timeout) ->
+    ?CATCH_EXIT(gen_server:call(?MODULE, code_path, Timeout)).
+
 code_status(TimeNative, TimeOffset, Timeout) ->
     ?CATCH_EXIT(gen_server:call(?MODULE,
                                 {code_status,
@@ -399,11 +412,12 @@ service_terminated(ID)
 %%% Callback functions from gen_server
 %%%------------------------------------------------------------------------
 
-init([Config]) ->
+init([TerminateTimeout, Config]) ->
     TimeStart = cloudi_timestamp:native_monotonic(),
     NodeName = cloudi_core_i_nodes:node_name(node()),
     {ok, #state{node_name = NodeName,
                 time_start = TimeStart,
+                terminate_timeout = TerminateTimeout,
                 configuration = Config}}.
 
 handle_call(configure, _, State) ->
@@ -623,6 +637,28 @@ handle_call(logging, _,
             #state{configuration = Config} = State) ->
     {reply, {ok, cloudi_core_i_configuration:logging(Config)}, State};
 
+handle_call({code_path_add, Path}, _,
+            #state{configuration = Config} = State) ->
+    case cloudi_core_i_configuration:code_path_add(Path, Config) of
+        {ok, ConfigNew} ->
+            {reply, ok, State#state{configuration = ConfigNew}};
+        {error, _} = Error ->
+            {reply, Error, State}
+    end;
+
+handle_call({code_path_remove, Path}, _,
+            #state{configuration = Config} = State) ->
+    case cloudi_core_i_configuration:code_path_remove(Path, Config) of
+        {ok, ConfigNew} ->
+            {reply, ok, State#state{configuration = ConfigNew}};
+        {error, _} = Error ->
+            {reply, Error, State}
+    end;
+
+handle_call(code_path, _,
+            #state{configuration = Config} = State) ->
+    {reply, {ok, cloudi_core_i_configuration:code_path(Config)}, State};
+
 handle_call({code_status, TimeNative, TimeOffset}, _,
             #state{time_start = TimeStart,
                    configuration = Config} = State) ->
@@ -677,7 +713,14 @@ handle_info({ReplyRef, _}, State) when is_reference(ReplyRef) ->
 handle_info(Request, State) ->
     {stop, cloudi_string:format("Unknown info \"~w\"", [Request]), State}.
 
-terminate(_, _) ->
+terminate(_Reason,
+          #state{terminate_timeout = Timeout,
+                 configuration = #config{code = ConfigCode}}) ->
+    #config_code{modules = Modules,
+                 applications = Applications} = ConfigCode,
+    _ = cloudi_x_reltool_util:modules_purged(Modules, Timeout),
+    _ = cloudi_x_reltool_util:applications_remove(Applications, Timeout,
+                                                  [cloudi_core]),
     ok.
 
 code_change(_, State, _) ->
