@@ -10,7 +10,7 @@
 %%%
 %%% MIT License
 %%%
-%%% Copyright (c) 2017-2018 Michael Truog <mjtruog at protonmail dot com>
+%%% Copyright (c) 2017-2020 Michael Truog <mjtruog at protonmail dot com>
 %%%
 %%% Permission is hereby granted, free of charge, to any person obtaining a
 %%% copy of this software and associated documentation files (the "Software"),
@@ -31,8 +31,8 @@
 %%% DEALINGS IN THE SOFTWARE.
 %%%
 %%% @author Michael Truog <mjtruog at protonmail dot com>
-%%% @copyright 2017-2018 Michael Truog
-%%% @version 1.7.4 {@date} {@time}
+%%% @copyright 2017-2020 Michael Truog
+%%% @version 1.8.1 {@date} {@time}
 %%%------------------------------------------------------------------------
 
 -module(cloudi_x_cpg_node_monitor).
@@ -80,10 +80,11 @@ stop_link(Process) ->
     ok.
 
 -spec died(Process :: process()) ->
-    list(pid()).
+    {list(pid()), list(nonempty_list({pid(), {exit, any()}}))}.
 
 died(Process) ->
-    monitors_flush(gen_server:call(Process, died)).
+    Pids = gen_server:call(Process, died),
+    {Pids, monitors_flush([])}.
 
 -spec add(Process :: process(),
           Pid :: pid()) ->
@@ -160,28 +161,26 @@ code_change(_, State, _) ->
 %%% Private functions
 %%%------------------------------------------------------------------------
 
-monitors_send({'DOWN', _MonitorRef, process, Pid, Info}, DOWNS,
+monitors_send({'DOWN', _MonitorRef, process, Pid, Info}, PidReasons,
               #state{parent = Parent,
                      monitors = Monitors} = State) ->
-    NewDOWNS = [{Pid, {exit, Info}} | DOWNS],
-    NewState = State#state{monitors = maps:remove(Pid, Monitors)},
+    PidReasonsNew = [{Pid, {exit, Info}} | PidReasons],
+    StateNew = State#state{monitors = maps:remove(Pid, Monitors)},
     receive
         {'DOWN', _, process, _, _} = DOWN ->
-            monitors_send(DOWN, NewDOWNS, NewState)
+            monitors_send(DOWN, PidReasonsNew, StateNew)
     after
-        ?MONITORS_ACCUMULATE_DELAY ->
-            Parent ! {'DOWNS', NewDOWNS},
-            NewState
+        ?MONITORS_SEND_DELAY ->
+            Parent ! {'DOWNS', PidReasonsNew},
+            StateNew
     end.
 
-monitors_flush(Pids) ->
+monitors_flush(PidReasonsL) ->
     receive
         {'DOWNS', PidReasons} ->
-            monitors_flush(lists:foldl(fun({Pid, _}, NewPids) ->
-                [Pid | NewPids]
-            end, Pids, PidReasons))
+            monitors_flush([PidReasons | PidReasonsL])
     after
-        0 ->
-            Pids
+        ?MONITORS_FLUSH_DELAY ->
+            lists:reverse(PidReasonsL)
     end.
 
