@@ -3,7 +3,7 @@
 //
 // MIT License
 //
-// Copyright (c) 2011-2017 Michael Truog <mjtruog at protonmail dot com>
+// Copyright (c) 2011-2021 Michael Truog <mjtruog at protonmail dot com>
 //
 // Permission is hereby granted, free of charge, to any person obtaining a
 // copy of this software and associated documentation files (the "Software"),
@@ -28,54 +28,58 @@
 #include "thread_pool.hpp"
 #include "piqpr8_gmp.hpp"
 #include "piqpr8_gmp_verify.hpp"
-#include <unistd.h>
+#include <boost/thread/mutex.hpp>
+#include <boost/thread/locks.hpp>
+#include <boost/thread/condition_variable.hpp>
 #include <sstream>
 #include <iostream>
 #include <string>
-#include <cstring>
 #include "assert.hpp"
 
-class ThreadData
-{
-};
-
-class OutputData
+class thread_data : public thread_pool_thread_data
 {
     public:
-        OutputData() : m_error(-1) {}
-        OutputData & setError(int value)
+        virtual ~thread_data() throw() {}
+};
+
+class task_output_data : public thread_pool_output_data
+{
+    public:
+        task_output_data(CloudI::API::return_value_type const error) :
+            m_error(error)
         {
-            m_error = value;
-            return *this;
         }
-        int error() const { return m_error; }
-        
+
+        virtual ~task_output_data() throw() {}
+
+        int error() const
+        {
+            int error_value = 0;
+            if (m_error && m_error != CloudI::API::return_value::terminate)
+                error_value = m_error;
+            return error_value;
+        }
+
     private:
-        int m_error;
+        CloudI::API::return_value_type const m_error;
 };
 
-class Input
+class task : public thread_pool_input<thread_data, task_output_data>
 {
     public:
-        Input(unsigned int const thread_index) :
+        task(unsigned int const thread_index) :
             m_stop_default(false),
             m_stop(m_stop_default),
-            m_api(thread_index)
+            m_api(thread_index, true)
         {
-            int result = 0;
-            result = m_api.subscribe_count("hexpi");
-            assert(result == CloudI::API::return_value::success);
-            assert(m_api.get_subscribe_count() == 0);
-            result = m_api.subscribe("hexpi", *this, &Input::hexpi);
-            assert(result == CloudI::API::return_value::success);
-            result = m_api.subscribe_count("hexpi");
-            assert(result == CloudI::API::return_value::success);
-            assert(m_api.get_subscribe_count() == 1);
+            task::m_timeout_terminate = m_api.timeout_terminate();
         }
 
-        uint32_t timeout_terminate() const
+        virtual ~task() throw() {}
+
+        static uint32_t timeout_terminate()
         {
-            return m_api.timeout_terminate();
+            return task::m_timeout_terminate;
         }
 
         void hexpi(CloudI::API const & api,
@@ -138,25 +142,44 @@ class Input
             std::cout << "execution never gets here" << std::endl;
         }
 
-        OutputData process(bool const & stop, ThreadData & /*data*/)
+        task_output_data process(bool const & stop, thread_data & /*data*/)
         {
-            OutputData resultObject;
-            int value;
-            m_stop = stop;
-            // a return from this Input object function, provides
-            // a single OutputData object (with the error value) to the
-            // single shared Output object, which will then cause the
+            // a return from this task object function, provides
+            // a single task_output_data object (with the error value) to the
+            // single shared task_output object, which will then cause the
             // thread pool to use the exit() function
             // (m_stop is set to true in all threads,
             //  so all threads are asked to abort their processing)
-            while (CloudI::API::return_value::timeout ==
-                   (value = m_api.poll(1000)))
+            m_stop = stop;
+            // it is best to do CloudI API subscribe calls here due to
+            // ownership of *this in the thread_pool
+            // when the process function is executed
+            // (makes it easier to use the thread_data variable,
+            //  if necessary,  because *this isn't getting copied)
+            int error = m_api.subscribe_count("hexpi");
+            if (! error)
             {
-                if (stop)
-                    return resultObject.setError(
-                        CloudI::API::return_value::success);
+                assert(m_api.get_subscribe_count() == 0);
+                error = m_api.subscribe("hexpi", *this, &task::hexpi);
             }
-            return resultObject.setError(value);
+            if (! error)
+            {
+                error = m_api.subscribe_count("hexpi");
+            }
+            if (! error)
+            {
+                assert(m_api.get_subscribe_count() == 1);
+                while (CloudI::API::return_value::timeout ==
+                       (error = m_api.poll(1000)))
+                {
+                    if (stop)
+                    {
+                        error = CloudI::API::return_value::success;
+                        break;
+                    }
+                }
+            }
+            return task_output_data(error);
         }
 
     private:
@@ -164,55 +187,91 @@ class Input
         bool & m_stop;
 
         CloudI::API m_api;
-
+        static uint32_t m_timeout_terminate;
 };
+uint32_t task::m_timeout_terminate = 0;
 
-class Output
+class task_output : public thread_pool_output<task_output_data>
 {
     public:
-        Output() : m_got_output(false) {}
-
-        void output(OutputData & data)
+        task_output() :
+            m_status(0),
+            m_terminate(false)
         {
-            if (data.error() &&
-                data.error() != CloudI::API::return_value::terminate)
-            {
-                std::cerr << "CloudI error " << data.error() << std::endl;
-            }
-            m_got_output = true;
         }
 
-        bool got_output() const { return m_got_output; }
+        virtual ~task_output() throw() {}
+
+        void output(task_output_data & data)
+        {
+            bool terminated;
+            {
+                boost::lock_guard<boost::mutex> lock(m_terminate_mutex);
+                int error = data.error();
+                if (error)
+                {
+                    std::cerr << "C/C++ CloudI API error " <<
+                        error << std::endl;
+                    m_status = error;
+                }
+                terminated = m_terminate;
+                m_terminate = true;
+            }
+            if (terminated == false)
+                m_terminate_conditional.notify_all();
+        }
+
+        int wait_on_terminate()
+        {   
+            if (m_terminate == false)
+            {
+                boost::unique_lock<boost::mutex> lock(m_terminate_mutex);
+                m_terminate_conditional.wait(lock);
+            }
+            return m_status;
+        }
 
     private:
-        bool m_got_output;
+        int m_status;
+        bool m_terminate;
+        boost::mutex m_terminate_mutex;
+        boost::condition_variable m_terminate_conditional;
 };
 
 int main(int, char **)
 {
-    unsigned int const thread_count = CloudI::API::thread_count();
+    int status = 0;
 
-    Output outputObject;
-    ThreadPool<Input, ThreadData, Output, OutputData>
-        threadPool(thread_count, thread_count, outputObject);
-
-    uint32_t timeout_terminate = 0;
-    for (unsigned int i = 0; i < thread_count; ++i)
+    assert_initialize();
+    try
     {
-        Input inputObject(i);
-        if (timeout_terminate == 0)
+        unsigned int const thread_count = CloudI::API::thread_count();
+
+        task_output output_object;
+        thread_pool<task, thread_data, task_output, task_output_data>
+            thread_pool(thread_count, thread_count, output_object);
+
+        for (unsigned int i = 0; i < thread_count; ++i)
         {
-            timeout_terminate = inputObject.timeout_terminate();
-            assert(timeout_terminate >= 1000);
+            task task_input(i);
+            bool const added = thread_pool.input(task_input);
+            assert(added);
         }
-        bool const result = threadPool.input(inputObject);
-        assert(result);
+
+        assert(task::timeout_terminate() >= 1000);
+        status = output_object.wait_on_terminate();
+        thread_pool.exit(task::timeout_terminate() - 100);
+    }
+    catch (CloudI::API::invalid_input_exception const & e)
+    {
+        std::cerr << e.what() << std::endl;
+    }
+    catch (CloudI::API::terminate_exception const &)
+    {
     }
 
-    while (outputObject.got_output() == false)
-        ::sleep(1);
     std::cout << "terminate hexpi c++" << std::endl;
-    threadPool.exit(timeout_terminate - 100);
-    return 0;
+
+    return status;
 }
 
