@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2017-2019 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2017-2021 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -38,9 +38,10 @@ class Task(threading.Thread):
     """
     null thread task
     """
-    def __init__(self, api, name, exception):
+    def __init__(self, thread_index, name, exception):
         threading.Thread.__init__(self)
-        self.__api = api
+        self.__api = None
+        self.__thread_index = thread_index
         self.__name = name
         self.__terminate_exception = exception
 
@@ -48,15 +49,15 @@ class Task(threading.Thread):
         """
         run the null thread
         """
-        # pylint: disable=bare-except
         try:
+            self.__api = API(self.__thread_index)
             self.__api.subscribe(self.__name + '/get', self.__request)
 
             result = self.__api.poll()
             assert result is False
         except self.__terminate_exception:
             pass
-        except:
+        except Exception:
             traceback.print_exc(file=sys.stderr)
         print('terminate null %s' % self.__name)
 
@@ -71,7 +72,7 @@ def _main():
     thread_count = API.thread_count()
     assert thread_count >= 1
 
-    threads = [Task(API(thread_index), 'python', TerminateException)
+    threads = [Task(thread_index, 'python', TerminateException)
                for thread_index in range(thread_count)]
     for thread in threads:
         thread.start()
