@@ -4,7 +4,7 @@
 #
 # MIT License
 #
-# Copyright (c) 2013-2019 Michael Truog <mjtruog at protonmail dot com>
+# Copyright (c) 2013-2021 Michael Truog <mjtruog at protonmail dot com>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -36,13 +36,14 @@ import traceback
 from cloudi_c import API, TerminateException
 
 class _Task(threading.Thread):
-    def __init__(self, api):
+    def __init__(self, thread_index):
         threading.Thread.__init__(self)
-        self.__api = api
+        self.__api = None
+        self.__thread_index = thread_index
 
     def run(self):
-        # pylint: disable=bare-except
         try:
+            self.__api = API(self.__thread_index)
             self.__api.subscribe('bounce/get', self.__request)
             self.__api.subscribe('bounce/delay', self.__delay)
             self.__api.subscribe('bounce/websocket/connect',
@@ -54,7 +55,7 @@ class _Task(threading.Thread):
             assert result is False
         except TerminateException:
             pass
-        except:
+        except Exception:
             traceback.print_exc(file=sys.stderr)
         print('terminate websockets python_c')
 
@@ -64,7 +65,7 @@ class _Task(threading.Thread):
         # pylint: disable=too-many-arguments
         assert request == b'CONNECT'
         print('connect: %s' %
-              str(self.__api.info_key_value_parse(request_info)))
+              str(API.info_key_value_parse(request_info)))
         return 'got connect! yay!'
 
     def __disconnect(self, request_type, name, pattern, request_info, request,
@@ -73,7 +74,7 @@ class _Task(threading.Thread):
         # pylint: disable=too-many-arguments
         assert request == b'DISCONNECT'
         print('disconnect: %s' %
-              str(self.__api.info_key_value_parse(request_info)))
+              str(API.info_key_value_parse(request_info)))
         return ''
 
     def __request(self, request_type, name, pattern, request_info, request,
@@ -106,7 +107,7 @@ def _main():
     thread_count = API.thread_count()
     assert thread_count >= 1
 
-    threads = [_Task(API(thread_index))
+    threads = [_Task(thread_index)
                for thread_index in range(thread_count)]
     for thread in threads:
         thread.start()
